var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
function resolveLocale(locales) {
    var localeData = UnifiedNumberFormat.__unitLocaleData__;
    var resolvedLocales = (Array.isArray(locales)
        ? locales
        : [locales]).filter(function (s) { return typeof s === 'string'; });
    var i, len, localeParts, data;
    var supportedLocales = [];
    // Using the set of locales + the default locale, we look for the first one
    // which that has been registered. When data does not exist for a locale, we
    // traverse its ancestors to find something that's been registered within
    // its hierarchy of locales. Since we lack the proper `parentLocale` data
    // here, we must take a naive approach to traversal.
    for (i = 0, len = resolvedLocales.length; i < len; i += 1) {
        localeParts = resolvedLocales[i].toLowerCase().split('-');
        while (localeParts.length) {
            data = localeData[localeParts.join('-')];
            if (data) {
                // Return the normalized locale string; e.g., we return "en-US",
                // instead of "en-us".
                supportedLocales.push(data.locale);
                break;
            }
            localeParts.pop();
        }
    }
    return supportedLocales;
}
export function isUnitSupported(unit) {
    try {
        new Intl.NumberFormat(undefined, {
            style: 'unit',
            unit: unit,
        });
    }
    catch (e) {
        return false;
    }
    return true;
}
var NativeNumberFormat = Intl.NumberFormat;
var UnifiedNumberFormat = /** @class */ (function () {
    function UnifiedNumberFormat(locale, _a) {
        if (_a === void 0) { _a = {}; }
        var style = _a.style, unit = _a.unit, unitDisplay = _a.unitDisplay, options = __rest(_a, ["style", "unit", "unitDisplay"]);
        this.unit = undefined;
        this.unitDisplay = undefined;
        if (style === 'unit') {
            if (!unit) {
                throw new TypeError('Unit is required for `style: unit`');
            }
            this.unit = unit;
            this.unitDisplay = unitDisplay || 'short';
            this.locale = resolveLocale([locale])[0];
            this.patternData = this.findUnitData(this.locale, this.unit);
        }
        this.nf = new NativeNumberFormat(locale, __assign({}, options, { style: style === 'unit' ? 'decimal' : style }));
        this.locale = this.nf.resolvedOptions().locale;
    }
    UnifiedNumberFormat.prototype.format = function (num) {
        var formattedNum = this.nf.format(num);
        if (this.patternData) {
            var pl = new Intl.PluralRules(this.locale).select(num);
            var pattern = this.patternData[this.unitDisplay][pl === 'one' ? 'one' : 'other'];
            return pattern.replace('{0}', formattedNum);
        }
        return formattedNum;
    };
    UnifiedNumberFormat.prototype.formatToParts = function (num) {
        return this.nf.formatToParts(num);
    };
    UnifiedNumberFormat.prototype.resolvedOptions = function () {
        var ro = this.nf.resolvedOptions();
        if (this.unit) {
            ro.style = 'unit';
            ro.unit = this.unit;
            ro.unitDisplay = this.unitDisplay;
        }
        return ro;
    };
    UnifiedNumberFormat.prototype.findUnitData = function (locale, unit) {
        var data = UnifiedNumberFormat.__unitLocaleData__;
        var parentLocale = '';
        locale = locale.toLowerCase();
        if (!data[locale]) {
            parentLocale = locale.split('-')[0];
        }
        else {
            if (data[locale].units[unit]) {
                return data[locale].units[unit];
            }
            if (data[locale].parentLocale) {
                parentLocale = data[locale].parentLocale;
            }
            else {
                throw new RangeError("Cannot find data for " + locale);
            }
        }
        return this.findUnitData(parentLocale, unit);
    };
    UnifiedNumberFormat.supportedLocalesOf = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return resolveLocale(NativeNumberFormat.supportedLocalesOf.apply(NativeNumberFormat, args));
    };
    UnifiedNumberFormat.__addUnitLocaleData = function (data) {
        Object.keys(data).forEach(function (locale) {
            var datum = data[locale];
            if (!(datum && datum.locale)) {
                throw new Error('Locale data provided to UnifiedNumberFormat is missing a ' +
                    '`locale` property value');
            }
            UnifiedNumberFormat.__unitLocaleData__[datum.locale.toLowerCase()] = datum;
        });
    };
    UnifiedNumberFormat.polyfilled = true;
    UnifiedNumberFormat.__unitLocaleData__ = {};
    return UnifiedNumberFormat;
}());
export default UnifiedNumberFormat;
